EXTSOCIALUI_VERSION = GetAddOnMetadata("[G]ExtSocialUI", "Version");
EXTSOCIALUI_VERSION_ID = 10002;
EXTSOCIALUI_DATA = {};

local EXT_LOADED = false;
local BLIZZ_RAID_LOADED = false;
local RAID_REBUILT = false;

local HOOKS = {};

local L = LibStub("AceLocale-3.0"):GetLocale("ExtSocialUI", true);

local _E;

local NEW_FRIEND_BUTTON_HEIGHT = 45;

--========================================
-- Initial load routine
--========================================
function ExtSocialUI_OnLoad(self)

    self:RegisterEvent("ADDON_LOADED");

    SLASH_EXTSOCIALUI1 = "/esui";
    SlashCmdList["EXTSOCIALUI"] = ExtSocialUI_CommandHandler;

end

--========================================
-- Event handler
--========================================
function ExtSocialUI_OnEvent(self, event, ...)
    
    if (event == "ADDON_LOADED") then
        local arg1 = ...;
        if (arg1 == "[G]ExtSocialUI") then
            ExtSocialUI_Setup();
            EXT_LOADED = true;
            ExtSocialUI_CheckLoadState();
        elseif (arg1 == "Blizzard_RaidUI") then
            BLIZZ_RAID_LOADED = true;
            ExtSocialUI_CheckLoadState();
        end
    end

end

--========================================
-- Post-load setup
--========================================
function ExtSocialUI_Setup()

    local version = ExtSocialUI_CheckSetting("version", EXTSOCIALUI_VERSION_ID);

    EXTSOCIALUI_DATA['config']['version'] = EXTSOCIALUI_VERSION_ID;

    ExtSocialUI_CheckSetting("show_load_message", false);

    ChannelFrame:HookScript("OnShow", ExtSocialUI_RebuildChatTab);

    HOOKS["ChannelRoster_Update"] = ChannelRoster_Update;
    ChannelRoster_Update = ExtSocialUI_ChannelRoster_Update;
    HOOKS["ChannelList_Update"] = ChannelList_Update;
    ChannelList_Update = ExtSocialUI_ChannelList_Update;
    ChannelList_SetScroll = function() end;
    HOOKS["FriendsFrame_UpdateFriends"] = FriendsFrame_UpdateFriends;
    FriendsFrameFriendsScrollFrame.update = ExtSocialUI_FriendsFrame_UpdateFriends;
    FriendsFrame_UpdateFriends = ExtSocialUI_FriendsFrame_UpdateFriends;

    ExtSocialUI_RebuildFrame();

    if (EXTSOCIALUI_DATA['config']['show_load_message']) then
        ExtSocialUI_Message(string.format(L["LOADED_MESSAGE"], EXTSOCIALUI_VERSION));
    end

end

--========================================
-- Checks if both Extended Social UI and
-- Blizzard's Raid UI addons are loaded,
-- and rebuilds the raid UI
--========================================
function ExtSocialUI_CheckLoadState()
    if (EXT_LOADED and BLIZZ_RAID_LOADED and not RAID_REBUILT) then
        ExtSocialUI_RebuildRaidUI();
        RAID_REBUILT = true;
    end
end

--========================================
-- Check configuration setting, and
-- initialize with default value if not
-- present
--========================================
function ExtSocialUI_CheckSetting(field, default)

    if (not EXTSOCIALUI_DATA['config']) then
        EXTSOCIALUI_DATA['config'] = {};
    end
    if (EXTSOCIALUI_DATA['config'][field] == nil) then
        EXTSOCIALUI_DATA['config'][field] = default;
    end
    return EXTSOCIALUI_DATA['config'][field];
end

--========================================
-- Rebuilds the social frame into
-- the extended design
--========================================
function ExtSocialUI_RebuildFrame()

    --override stock values
    FRIENDS_BUTTON_NORMAL_HEIGHT = NEW_FRIEND_BUTTON_HEIGHT;
    FRIENDS_FRAME_FRIEND_HEIGHT = NEW_FRIEND_BUTTON_HEIGHT;
    FriendsFrameFriendsScrollFrame.buttonHeight = NEW_FRIEND_BUTTON_HEIGHT;

    local i;

    FriendsFrame:SetWidth(600);

    -- ********** Friends Tab **********

    FriendsFrameBattlenetFrame:ClearAllPoints();
    FriendsFrameBattlenetFrame:SetPoint("TOP", FriendsTabHeader, "TOP", 0, -27);
    FriendsFrameStatusDropDown:ClearAllPoints();
    FriendsFrameStatusDropDown:SetPoint("TOPLEFT", FriendsTabHeader, "TOPLEFT", 140, -27);

    FriendsFrameFriendsScrollFrame:SetWidth(564);
    for i = 1, 11, 1 do
        local btn = _G["FriendsFrameFriendsScrollFrameButton" .. i];
        if (btn) then
            btn:SetWidth(562);
            btn:SetHeight(NEW_FRIEND_BUTTON_HEIGHT);
            local nameText = _G["FriendsFrameFriendsScrollFrameButton" .. i .. "Name"];
            nameText:SetWidth(500);
            local infoText = _G["FriendsFrameFriendsScrollFrameButton" .. i .. "Info"];
            infoText:SetWidth(500);
            local noteIcon = btn:CreateTexture("FriendsFrameFriendsScrollFrameButton" .. i .. "NoteIcon", "ARTWORK");
            noteIcon:SetWidth(14);
            noteIcon:SetHeight(12);
            noteIcon:SetPoint("TOPLEFT", infoText, "BOTTOMLEFT", 0, -2);
            noteIcon:SetTexture("Interface/FriendsFrame/UI-FriendsFrame-Note");
            local noteText = btn:CreateFontString("FriendsFrameFriendsScrollFrameButton" .. i .. "Note", "ARTWORK", "FriendsFont_Small");
            noteText:SetPoint("LEFT", noteIcon, "RIGHT", 1, 0);
            noteText:SetWidth(484);
            noteText:SetJustifyH("LEFT");
        end
    end

    FriendsFrameIgnoreScrollFrame:ClearAllPoints();
    FriendsFrameIgnoreScrollFrame:SetPoint("TOPRIGHT", FriendsFrame, "TOPRIGHT", -32, -86);
    FriendsFrameIgnoreScrollFrame:SetWidth(560);
    FriendsFrameIgnoreScrollFrame:SetHeight(311);
    for i = 1, 20, 1 do
        local btn = _G["FriendsFrameIgnoreButton" .. i];
        if (btn) then
            btn:SetWidth(564);
        end
    end

    FriendsFramePendingScrollFrame:SetWidth(560);

    for i = 1, 5, 1 do
        local btn = _G["FriendsFramePendingButton" .. i];
        if (btn) then
            local nameBg = _G["FriendsFramePendingButton" .. i .. "Background"];
            local msg = _G["FriendsFramePendingButton" .. i .. "Message"];
            local accBtn = _G["FriendsFramePendingButton" .. i .. "AcceptButton"];
            nameBg:SetWidth(554);
            btn:SetWidth(562);
            msg:SetWidth(524);
            accBtn:ClearAllPoints();
            accBtn:SetPoint("TOPRIGHT", msg, "BOTTOM", -22, -10);
        end
    end

    -- ********** Who Tab **********

    WhoListScrollFrame:SetWidth(560);

    WhoFrameColumn_SetWidth(WhoFrameColumnHeader1, 90);

    -- change column 2 to zone
    WhoFrameDropDown:Hide();
    WhoFrameColumn_SetWidth(WhoFrameColumnHeader2, 130);
    WhoFrameColumnHeader2:SetText(ZONE);
    WhoFrameColumnHeader2.sortType = "zone";

    -- change column 3 to guild
    WhoFrameColumn_SetWidth(WhoFrameColumnHeader3, 160);
    WhoFrameColumnHeader3:SetText(GUILD);
    WhoFrameColumnHeader3.sortType = "guild";

    -- change column 4 to level
    WhoFrameColumn_SetWidth(WhoFrameColumnHeader4, 32);
    WhoFrameColumnHeader4:SetText(LEVEL_ABBR);
    WhoFrameColumnHeader4.sortType = "level";

    -- create column 5 and set it to race
    WhoFrameColumnHeader5 = CreateFrame("Button", "WhoFrameColumnHeader5", WhoFrame, "WhoFrameColumnHeaderTemplate");
    WhoFrameColumnHeader5:SetPoint("LEFT", WhoFrameColumnHeader4, "RIGHT", -2, 0);
    WhoFrameColumn_SetWidth(WhoFrameColumnHeader5, 70);
    WhoFrameColumnHeader5:SetText(RACE);
    WhoFrameColumnHeader5.sortType = "race";

    -- create column 6 and set it to class
    WhoFrameColumnHeader6 = CreateFrame("Button", "WhoFrameColumnHeader6", WhoFrame, "WhoFrameColumnHeaderTemplate");
    WhoFrameColumnHeader6:SetPoint("LEFT", WhoFrameColumnHeader5, "RIGHT", -2, 0);
    WhoFrameColumn_SetWidth(WhoFrameColumnHeader6, 85);
    WhoFrameColumnHeader6:SetText(CLASS);
    WhoFrameColumnHeader6.sortType = "class";

    -- create new who frame list buttons
    for i = 1, 17, 1 do
        local btn = _G["WhoFrameButton" .. i];
        if (btn) then
            btn:Hide();
        end

        local btn2 = CreateFrame("Button", "NewWhoFrameButton" .. i, WhoFrame, "ExtSocialUI_WhoButtonTemplate");
        btn2:SetID(i);
        if (i == 1) then
            btn2:SetPoint("TOPLEFT", WhoFrame, "TOPLEFT", 6, -82);
        else
            btn2:SetPoint("TOP", _G["NewWhoFrameButton" .. (i - 1)], "BOTTOM");
        end
    end

    WhoFrameEditBox:SetWidth(560);

    -- then hook the who list update function
    WhoList_Update = ExtSocialUI_WhoList_Update;

    -- ********** Raid Tab **********

    RaidFrameRaidDescription:SetWidth(566);

    -- force update the friends list
    FriendsList_Update();

end

--========================================
-- Rebuild the Chat/Channels tab
--========================================
function ExtSocialUI_RebuildChatTab()
    local i;

    ChannelFrame:SetWidth(600);
    ChannelFrameLeftInset:ClearAllPoints();
    ChannelFrameLeftInset:SetPoint("TOPLEFT", ChannelFrame, "TOPLEFT", 4, -60);
    ChannelFrameLeftInset:SetPoint("BOTTOMRIGHT", ChannelFrame, "BOTTOM", 0, 52);
    ChannelFrameRightInset:ClearAllPoints();
    ChannelFrameRightInset:SetPoint("TOPRIGHT", ChannelFrame, "TOPRIGHT", -8, -60);
    ChannelFrameRightInset:SetPoint("BOTTOMLEFT", ChannelFrame, "BOTTOM", 0, 52);

    ChannelListScrollFrame:SetWidth(290);

    for i = 1, 16, 1 do
        local btn = _G["ChannelButton" .. i];
        if (btn) then
            btn:SetWidth(290);
        end
    end

    ChannelRoster:SetWidth(290);
    ChannelRoster:ClearAllPoints();
    ChannelRoster:SetPoint("TOPLEFT", ChannelListScrollFrame, "TOPRIGHT", 6, 0);
    ChannelRosterScrollFrame:ClearAllPoints();
    ChannelRosterScrollFrame:SetPoint("TOPLEFT", ChannelRoster, "TOPLEFT");
    ChannelRosterScrollFrame:SetWidth(264);
    ChannelRosterChannelName:ClearAllPoints();
    ChannelRosterChannelName:SetPoint("TOPLEFT", ChannelRoster, "TOPLEFT", 0, 20);

    ChannelMemberButton1:ClearAllPoints();
    ChannelMemberButton1:SetPoint("TOPLEFT", ChannelRosterScrollFrame, "TOPLEFT");

    ExtSocialUI_ResizeChannelRosterButtons();

end

--========================================
-- Rebuild the Raid tab
--========================================
function ExtSocialUI_RebuildRaidUI()

    local i, j;

    for i = 1, 8, 1 do
        local btn = _G["RaidGroup" .. i];
        if (btn) then
            btn:SetWidth(292);
            ExtSocialUI_HideUnnamedTextures(btn);
            local btnBg = btn:CreateTexture("RaidGroup" .. i .. "Background", "BACKGROUND");
            btnBg:SetTexture("Interface\\RaidFrame\\UI-RaidFrame-GroupOutline");
            btnBg:SetTexCoord(0, 0.6640625, 0, 0.625);
            btnBg:SetPoint("TOPLEFT", btn, "TOPLEFT");
            btnBg:SetPoint("BOTTOMRIGHT", btn, "BOTTOMRIGHT");

            for j = 1, 5, 1 do
                local slot = _G["RaidGroup" .. i .. "Slot" .. j];
                if (slot) then
                    slot:SetWidth(286);
                end
            end
        end
    end

    for i = 1, 40, 1 do
        local btn = _G["RaidGroupButton" .. i];
        if (btn) then
            btn:SetWidth(286);
            local name = _G["RaidGroupButton" .. i .. "Name"];
            name:SetWidth(155);
        end
    end

end

--========================================
-- Hides unnamed texture elements on the
-- specified parent frame
--========================================
function ExtSocialUI_HideUnnamedTextures(parent)
    local list = { parent:GetRegions() };
    for i, j in pairs(list) do
        if (j:IsObjectType("Texture") and not j:GetName()) then
            j:Hide();
        end
    end
end

--========================================
-- Toggles a boolean config setting
--========================================
function ExtSocialUI_ToggleSetting(name)
    if (EXTSOCIALUI_DATA['config'][name]) then
        EXTSOCIALUI_DATA['config'][name] = false;
    else
        EXTSOCIALUI_DATA['config'][name] = true;
    end
end

--========================================
-- Output message to chat frame
--========================================
function ExtSocialUI_Message(msg)
    DEFAULT_CHAT_FRAME:AddMessage("|cffe000ff<" .. L["ADDON_TITLE"] .. ">|r " .. msg);
end

--========================================
-- Slash command handler
--========================================
function ExtSocialUI_CommandHandler(cmd)

    InterfaceOptionsFrame_OpenToCategory(ExtSocialUIConfigContainer);

end


--========================================
-- Hook for updating the /who list
--========================================
function ExtSocialUI_WhoList_Update()
	local numWhos, totalCount = GetNumWhoResults();
	local name, guild, level, race, class, zone;
	local button, buttonText, classTextColor, classFileName;
	local whoOffset = FauxScrollFrame_GetOffset(WhoListScrollFrame);
	local whoIndex;
	local showScrollBar = nil;

	if (numWhos > WHOS_TO_DISPLAY) then
		showScrollBar = 1;
	end

	local displayedText = "";
	if ( totalCount > MAX_WHOS_FROM_SERVER ) then
		displayedText = format(WHO_FRAME_SHOWN_TEMPLATE, MAX_WHOS_FROM_SERVER);
	end

	WhoFrameTotals:SetText(format(WHO_FRAME_TOTAL_TEMPLATE, totalCount) .. "  " .. displayedText);
	for i = 1, WHOS_TO_DISPLAY, 1 do
		whoIndex = whoOffset + i;
		button = _G["NewWhoFrameButton" .. i];
		button.whoIndex = whoIndex;
		name, guild, level, race, class, zone, classFileName = GetWhoInfo(whoIndex);

		if ( classFileName ) then
			classTextColor = RAID_CLASS_COLORS[classFileName];
		else
			classTextColor = HIGHLIGHT_FONT_COLOR;
		end
		buttonText = _G["NewWhoFrameButton"..i.."Name"];
		buttonText:SetText(name);
		buttonText = _G["NewWhoFrameButton"..i.."Zone"];
		buttonText:SetText(zone);
		buttonText = _G["NewWhoFrameButton"..i.."Guild"];
		buttonText:SetText(guild);
		buttonText = _G["NewWhoFrameButton"..i.."Level"];
		buttonText:SetText(level);
		buttonText = _G["NewWhoFrameButton"..i.."Race"];
		buttonText:SetText(race);
		buttonText = _G["NewWhoFrameButton"..i.."Class"];
		buttonText:SetText(class);
		buttonText:SetTextColor(classTextColor.r, classTextColor.g, classTextColor.b);
		
		-- Highlight the correct who
		if ( WhoFrame.selectedWho == whoIndex ) then
			button:LockHighlight();
		else
			button:UnlockHighlight();
		end
		
		if ( whoIndex > numWhos ) then
			button:Hide();
		else
			button:Show();
		end
	end

	if ( not WhoFrame.selectedWho ) then
		WhoFrameGroupInviteButton:Disable();
		WhoFrameAddFriendButton:Disable();
	else
		WhoFrameGroupInviteButton:Enable();
		WhoFrameAddFriendButton:Enable();
		WhoFrame.selectedName = GetWhoInfo(WhoFrame.selectedWho); 
	end

	-- ScrollFrame update
	FauxScrollFrame_Update(WhoListScrollFrame, numWhos, WHOS_TO_DISPLAY, FRIENDS_FRAME_WHO_HEIGHT );

	PanelTemplates_SetTab(FriendsFrame, 2);
	ShowUIPanel(FriendsFrame);
end


--========================================
-- Hook for Chat tab update (1)
--========================================
function ExtSocialUI_ChannelList_Update(id)
    HOOKS["ChannelList_Update"](id);
    ExtSocialUI_ResizeChannelRosterButtons();
end

--========================================
-- Hook for Chat tab update (2)
--========================================
function ExtSocialUI_ChannelRoster_Update(id)
    HOOKS["ChannelRoster_Update"](id);
    ExtSocialUI_ResizeChannelRosterButtons();
end

--========================================
-- Resize Chat tab roster list buttons
--========================================
function ExtSocialUI_ResizeChannelRosterButtons()
    local i;
    for i = 1, 22, 1 do
        local btn = _G["ChannelMemberButton" .. i];
        if (btn) then
            btn:SetWidth(264);
        end
    end
end

--========================================
-- Update friends list entries with notes
--========================================
function ExtSocialUI_FriendsFrame_UpdateFriends()

    HOOKS["FriendsFrame_UpdateFriends"]();

    if (FriendsFrame.selectedTab == 1) then
        local i;
        for i = 1, 11 do
            local btn = _G["FriendsFrameFriendsScrollFrameButton" .. i];
            if (btn) then
                local noteString = _G["FriendsFrameFriendsScrollFrameButton" .. i .. "Note"];
                local noteIcon = _G["FriendsFrameFriendsScrollFrameButton" .. i .. "NoteIcon"];
                local note = "";
                local isOnline = false;
                if (btn:IsVisible()) then
                    if (btn.buttonType == FRIENDS_BUTTON_TYPE_WOW) then
                        local name, level, class, area, connected, status, noteText = GetFriendInfo(btn.id);
                        note = noteText;
                        isOnline = connected;
                    elseif (btn.buttonType == FRIENDS_BUTTON_TYPE_BNET) then
                        local presenceID, presenceName, battleTag, isBattleTagPresence, toonName, toonID, client, connected, lastOnline, isAFK, isDND, broadcastText, noteText,
                        isRIDFriend, broadcastTime, canSoR = BNGetFriendInfo(btn.id);
                        note = noteText;
                        isOnline = connected;
                    end
                end
                note = (note or "");
                noteString:SetText(note);
                if (isOnline) then
                    noteString:SetTextColor(1, 0.82, 0);
                    noteIcon:SetVertexColor(1, 0.82, 0);
                else
                    noteString:SetTextColor(0.75, 0.75, 0.75);
                    noteIcon:SetVertexColor(0.75, 0.75, 0.75);
                end
            end
        end
    end

end
